#!/bin/bash
#==============================================================================
# Copyright and license info is available in the LICENSE file included with
# the Server Deployment Package (SDP), and also available online:
# https://swarm.workshop.perforce.com/projects/perforce-software-sdp/view/main/LICENSE
#------------------------------------------------------------------------------

# Intended to be run on a replica machine to sync replica from its corresponding master
# Like sync_replica.sh but for use in a regular crontab
export SDP_INSTANCE=${SDP_INSTANCE:-Undefined} 
export SDP_INSTANCE=${1:-$SDP_INSTANCE} 
if [[ $SDP_INSTANCE == Undefined ]]; then 
   echo "Instance parameter not supplied." 
   echo "You must supply the Perforce instance as a parameter to this script." 
   exit 1 
fi 
. /p4/common/bin/p4_vars $SDP_INSTANCE
. /p4/common/bin/backup_functions.sh

######### Start of Script ##########
check_vars
set_vars

# override setting of checkpoint.log from set_vars
LOGFILE=$LOGS/sync_replica.log

check_uid

$P4BIN -u $P4USER -p ${P4MASTERPORT} login < /p4/common/bin/adminpass > /dev/null 2>&1
MASTERJOURNALNUM=$($P4BIN -u $P4USER -p ${P4MASTERPORT} counter journal)

if [[ "$MASTERJOURNALNUM" == "" ]]; then
   die "Error:  Couldn't get journal number from master.  Aborting."
fi

# We set JOURNALNUM to one less than the master since we are not truncating the journal
# and replay_journals_to_offline_db assumes that truncate_journal has been run.

JOURNALNUM=$(($MASTERJOURNALNUM-1))

rotate_last_run_logs
log "Starting weekly_sync_replica.sh"

# You must set up a public keypair using "ssh-keygen -t rsa" in order for this to work.
# You need to paste your CLIENT ~/.ssh/id_rsa.pub contents into the REMOTE ~/ssh/authorized_keys file. 
if [[ "$SHAREDDATA" == "FALSE" ]]; then
   rsync -avz --delete ${OSUSER}@${P4MASTER}:$CHECKPOINTS/ $CHECKPOINTS > $LOGFILE 2>&1
   rsync_exit_code=$?

   if [[ $rsync_exit_code -ne 0 ]]; then
      die "Error: Failed to pull $CHECKPOINTS from host $P4MASTER.  The rsync exit code was: $rsync_exit_code.  Aborting."
   fi
fi

recreate_offline_db_files
get_offline_journal_num
replay_journals_to_offline_db

/p4/${SDP_INSTANCE}/bin/p4d_${SDP_INSTANCE}_init stop >> $LOGFILE 2>&1
exit_code=$?

if [[ $exit_code -ne 0 ]]; then
   die "Error: Server failed to shut down.  Aborting."
fi

log "Server should be down now."
log "Moving offline db into root."
cd /p4/${SDP_INSTANCE}/root
rm -f save/db.* >> $LOGFILE 2>&1
mv db.* save >> $LOGFILE 2>&1
mv /p4/${SDP_INSTANCE}/offline_db/db.* . >> $LOGFILE 2>&1
rm -f state* >> $LOGFILE 2>&1
rm -f rdb.lbr >> $LOGFILE 2>&1
rm -f /p4/$SDP_INSTANCE/logs/journal >> $LOGFILE 2>&1
rm -f /p4/$SDP_INSTANCE/checkpoints.rep/* >> $LOGFILE 2>&1

$P4BIN -p ${P4MASTERPORT} login < /p4/common/bin/adminpass > /dev/null 2>&1
$P4BIN -p ${P4MASTERPORT} login $P4SERVICEUSER > /dev/null 2>&1
/p4/${SDP_INSTANCE}/bin/p4d_${SDP_INSTANCE}_init start >> $LOGFILE 2>&1

log "Server should be back up now."

$P4BIN login < /p4/common/bin/adminpass > /dev/null 2>&1
$P4BIN pull -lj >> $LOGFILE

rm -f save/db.* >> $LOGFILE 2>&1
recreate_offline_db_files
get_offline_journal_num
replay_journals_to_offline_db

check_disk_space
remove_old_logs
export CHECKPOINTS=${P4HOME}/checkpoints.rep
remove_old_checkpoints_and_journals

$P4BIN pull -lj >> $LOGFILE

log "End $P4SERVER weekly sync replica log."
mail_log_file "$HOSTNAME $P4SERVER Weekly sync replica log."
